/**************************************************************************//**
*   @file   ad5668.c
*   @brief  Implementation of AD5668 Driver for Microblaze processor.
*   @author grohmast
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
#define AD5668_GLOBALS

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "ad5668.h"
#include "../drivers/spi.h"
#include "../drivers/gpio.h"
#include "../drivers/timer.h"

/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/
unsigned char   txBuffer[4] = {0,};         // Tx SPI Buffer.
unsigned char   rxBuffer[4] = {0,};         // Rx SPI Buffer.

/*****************************************************************************/
/*  Aliases for the uC Probe data                                            */
/*****************************************************************************/
//flag used to either read one channel with the ADC (loop=0) or all channels sequentially (loop=1)
volatile char*              loop            = &AD5668_Data[0].chData;
//if 1 the value will be updated on the DAC
volatile char*              setValue        = &AD5668_Data[1].chData;
//used to command the LDAC pin
volatile char*              ldac            = &AD5668_Data[2].chData;
//used to command the CLR pin
volatile char*              clr             = &AD5668_Data[3].chData;
//if 1, the command will be sent to the DAC
volatile char*              setCommand      = &AD5668_Data[4].chData;
//used as a flag to indicate that the command was not sent because of an error
volatile char*              cmdErr          = &AD5668_Data[5].chData;
//used to set the DAC value
volatile unsigned short*    dacValue        = &AD5668_Data[6].usiData;
//used to select the channel
volatile unsigned char*     dacChanNumber   = &AD5668_Data[7].uchData;
//used to send commands to the DAC. Will store the command number
volatile unsigned char*     cmdNumber       = &AD5668_Data[8].uchData;
//used to send commands to the DAC. Will store the address (channel)
volatile unsigned char*     cmdChannel      = &AD5668_Data[9].uchData;
//used to send commands to the DAC. Will store the value
volatile unsigned short*    cmdData         = &AD5668_Data[10].usiData;
//used for computing the ADC voltage
volatile float*             dvdd            = &AD5668_Data[11].fData;
//used for computing the DAC voltage
volatile float*             vref            = &AD5668_Data[12].fData;
//used for displaying the voltage to be set in uC-Probe
volatile float*             vset            = &AD5668_Data[13].fData;

/**************************************************************************//**
 * @brief get OnBoard ADC data.
 *
 * @param none
 *
 * @return ADC data
******************************************************************************/
int AD5668_GetAd7475(void)
{
    int returnval;
    AD5668_Mux(MUX_TARGET_ADC); // enable ADC comm
    txBuffer[0] = (unsigned char) 0;
    txBuffer[1] = (unsigned char) 0;
    SPI_TransferData(AD5668_SPI, 0, NULL, 2, (char *) rxBuffer, 1);
    TIMER0_WAIT(XPAR_AXI_TIMER_0_BASEADDR, 10000);//wait 10us
    // power mode!
    SPI_TransferData(AD5668_SPI, 0, NULL, 2, (char *) rxBuffer, 1);
    returnval  = ((int) rxBuffer[0]) << 8;
    returnval +=  (int) rxBuffer[1];

    return(returnval);
}

/**************************************************************************//**
 * @brief set OnBoard MUX
 *
 * @param value - MUX data
 *
 * @return TRUE
******************************************************************************/
int AD5668_SetAdg738(int value)
{
    AD5668_Mux(MUX_TARGET_MUX);         // enable MUX comm
    txBuffer[0] = (unsigned char) (value & 0xff);
    SPI_TransferData(AD5668_SPI, 1, (char * )txBuffer, 0, NULL, 1);

    return(TRUE);
}

/**************************************************************************//**
 * @brief set OnBoard DAC data.
 *
 * @param value - DAC data
 *
 * @return TRUE
******************************************************************************/

int AD5668_Set(int value)
{
    txBuffer[0] = (unsigned char) ((value & 0x0f000000) >> 24);
    txBuffer[1] = (unsigned char) ((value & 0x00ff0000) >> 16);
    txBuffer[2] = (unsigned char) ((value & 0x0000ff00) >> 8);
    txBuffer[3] = (unsigned char)  (value & 0x000000ff);
    SPI_TransferData(AD5668_SPI, 4, (char *) txBuffer, 0, NULL, 1);

    return(TRUE);
}

/**************************************************************************//**
 * @brief Toggle clear data pin
 *
 * @param none
 *
 * @return none
******************************************************************************/
void AD5668_Clear(void)
{
    // active falling edge
    GPIO_CLEAR_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_DAC_CLR_N);   // set low
    TIMER0_WAIT(XPAR_AXI_TIMER_0_BASEADDR, 10000);//wait 10us
    GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_DAC_CLR_N);     // set high
}

/**************************************************************************//**
 * @brief communication MUX
 *
 * @param target
 *
 * @return none
******************************************************************************/
void AD5668_Mux(int target)
{
    int data;
    data = GPIO_GET_DATA(XPAR_AXI_GPIO_0_BASEADDR);
    data |= (GPIO_DAC|GPIO_ADC|GPIO_MUX);
    data &= ~(1<<(target+2));
    GPIO_SET_DATA(XPAR_AXI_GPIO_0_BASEADDR, data);
}

/**************************************************************************//**
 * @brief Toggle LDAC pin
 *
 * @param none
 *
 * @return none
******************************************************************************/
void AD5668_Ldac(void)
{
    // active rising edge
    GPIO_CLEAR_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_LDAC_N);  // set low
    TIMER0_WAIT(XPAR_AXI_TIMER_0_BASEADDR, 10000);//wait 10us
    GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_LDAC_N);    // set high
}

/**************************************************************************//**
 * @brief Initialize all channels with power mode normal
 *
 * @param none
 *
 * @return none
******************************************************************************/
void AD5668_SetPor(void)
{
    int value;
    // reset
    AD5668_Mux(MUX_TARGET_DAC);
    value =  (RESET<<AD5668CMDPOSITION);
    AD5668_Set(value);
    // power on
    value =  (POWER_DOWN_POWER_UP_DAC<<AD5668CMDPOSITION);
    value += (PWR_NORM<<AD5668PWRPOSITION);
    value += CHANNEL_ALL;
    AD5668_Set(value);
}

/**************************************************************************//**
 * @par Description:
 *  Set the internal reference voltage on
 *
 * @param none
 *
 * @return none
******************************************************************************/
void AD5668_SetRef(void)
{
    int value;
    AD5668_Mux(MUX_TARGET_DAC);
    value =  (SET_UP_INTERNAL_REF_REGISTER<<AD5668CMDPOSITION);
    value += INTERN_REF_ON;
    AD5668_Set(value);
}

/**************************************************************************//**
 * @par Description:
 * Initialize the evaluation board:
 *  The initial reference voltage is considered 3.3 (evaluation board powered from the BeMicro)
 *  All input variables are set to 0*
 *
 * @param none
 *
 * @return TRUE if it the read from SPI returns a value different than 0xffff
 * @return FALSE in case the read from SPI returns 0xffff
******************************************************************************/
int AD5668_Init(void)
{
    // enable output buffers of SPORT SPI interface, inverted
    SPI_Init(AD5668_SPI, 0, 0, 1);      // Setup the SPI interface

    GPIO_SET_OUTPUT_PINS(XPAR_AXI_GPIO_0_BASEADDR, ~(   GPIO_DAC_CLR_N  \
                                                        | GPIO_LDAC_N   \
                                                        | GPIO_DAC      \
                                                        | GPIO_ADC      \
                                                        | GPIO_MUX ));       // set output
    AD5668_Mux(MUX_TARGET_DAC);                                             // activate DAC communication
    GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_DAC_CLR_N);             // set CLR HIGH, active LOW
    GPIO_SET_BITS(XPAR_AXI_GPIO_0_BASEADDR, GPIO_LDAC_N);                // set LDAC HIGH, active LOW
    if (AD5668_GetAdc(0) != 65535 )
    {
        *cmdErr  = 0;
        *dvdd    = 5.0f;
        *vref    = 5.0f;
        return(TRUE);
    }
    else
    {
        return(FALSE);
    }
}

/**************************************************************************//**
 * @brief Function used to read the voltage on different channels
 *
 * @param channel - sets the channel to be read (valid values 0 to 7)
 *
 * @return The ADC code corresponding to the selected channel
******************************************************************************/
int AD5668_GetAdc(int channel)
{
    int adcValue;
    if (channel > 7)
    {
        channel = 0;
    }
    // setup ADC channel MUX
    AD5668_SetAdg738(1<<channel);
    // get ADC value
    adcValue = AD5668_GetAd7475();
    return (adcValue);
}

/**************************************************************************//**
 * @brief Used to set the out value on the DAC
 *
 * @param channel - Used to select the channel
 * @param dac - Used to set the output value
 *
 * @return none
******************************************************************************/
void AD5668_SetDac(int channel, int dac)
{
    int value;
    if (dac > 0xFFFF) {
        dac = 0xFF;
    }
    if (dac < 0) {
        dac = 0;
    }
    channel = (channel & 0x0F);
    AD5668_Mux(MUX_TARGET_DAC);
    value =  (WRITE_TO_REGISTER_AND_UPDATE_CHANNEL_N<<AD5668CMDPOSITION);
    value += (channel<<AD5668CHANPOSITION);
    value += (dac<<AD5668DATAPOSITION);
    AD5668_Set(value);
}

/**************************************************************************//**
 * @brief Function used to send a specific command to the AD5668
 *
 * @param command - Parameter used to indicate the command to be sent
 * @param channel - Parameter used to select the channel on which to apply the command
 * @param dac - Parameter used to set the databits of the command
 *
 * @return TRUE/FALSE.
******************************************************************************/
int AD5668_SetDacCommand(char command,char channel, unsigned short dac)
{
    int value = 0;
    channel = (channel & 0x0F);
    AD5668_Mux(MUX_TARGET_DAC);
    if (command==WRITE_TO_INPUT_REGISTER_N)             //command 0 Write Input Register n
    {
        if (channel>7&&channel!=0xF) return(FALSE);
        value =  (WRITE_TO_INPUT_REGISTER_N<<AD5668CMDPOSITION);
        value += (channel<<AD5668CHANPOSITION);         //select which register to write
        value += (dac<<AD5668DATAPOSITION);             //the value to be written
    }
    if (command==UPDATE_DAC_REGISTER_N)                 //command 1 Update DAC Register n
    {
        if (channel>7&&channel!=0xF) return(FALSE);
        value =  (UPDATE_DAC_REGISTER_N<<AD5668CMDPOSITION);
        value += (channel<<AD5668CHANPOSITION);         //select for which register the DAC value should be updated
        //the value is not important, because it will use the value set with command 0.
        value += (dac<<AD5668DATAPOSITION);
    }
    // command 2 Write to Input Register n, update all (software \LDAC)
    if (command==WRITE_TO_INPUT_REGISTER_N_UPDATE_ALL)
    {
        if (channel>7&&channel!=0xF) return(FALSE);
        value =  (WRITE_TO_INPUT_REGISTER_N_UPDATE_ALL<<AD5668CMDPOSITION);
        value += (channel<<AD5668CHANPOSITION);         //select for which port the DAC value should be updated
        value += (dac<<AD5668DATAPOSITION);             //the new value
    }
    if (command==WRITE_TO_REGISTER_AND_UPDATE_CHANNEL_N)//command 3 Write and update DAC channel n
    {
        if (channel>7&&channel!=0xF) return(FALSE);
        value =  (WRITE_TO_REGISTER_AND_UPDATE_CHANNEL_N<<AD5668CMDPOSITION);
        value += (channel<<AD5668CHANPOSITION);         //Select the channel to be updated
        value += (dac<<AD5668DATAPOSITION);             //The new value to be logged
    }
    if (command==POWER_DOWN_POWER_UP_DAC)               //command 4 Power down/power up DAC
    {
        dac=dac&0xFF;                                   //dac used for channel selection. Only 8 bits needed
        value =  (POWER_DOWN_POWER_UP_DAC<<AD5668CMDPOSITION);
        value += (channel<<AD5668PWRPOSITION);          //Selects the power down mode
        value += (dac);                                 //selects the channels
    }
    if (command==LOAD_CLEAR_CODE_REGISTER)              //command 5 Load clear code register
    {

        value =  (LOAD_CLEAR_CODE_REGISTER<<AD5668CMDPOSITION);
        value += (dac);//dac used for Clear code register value. This will be used on all channels
    }
    if (command==LOAD_LDAC_REGISTER)//command 6 Load \LDAC register
    {

        value =  (LOAD_LDAC_REGISTER<<AD5668CMDPOSITION);
        //dac used for channel selections. Channels for which the value is 1 will consider always \LDAC active
        value += (dac);
    }
    //command 7 Power on Reset. If using internal reference, the command should be resent after a reset
    if (command==RESET)
    {
        value =  (RESET<<AD5668CMDPOSITION);
    }
    if (command==SET_UP_INTERNAL_REF_REGISTER)          //command 8 Set up internal REF register
    {
        if (dac)dac=1;
        value =  (SET_UP_INTERNAL_REF_REGISTER<<AD5668CMDPOSITION);
        //dac used for setting the internal reference; 0 internal reference off, other: internal reference on;
        value += (dac);
    }
    AD5668_Set(value);

    return(TRUE);
}

/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters.
 *
 * @param None.
 *
 * @return TRUE.
******************************************************************************/
int AD5668_UpdateParameters(void)
{
    int     i = 0;
    float   tempVRef;       //used as intermediary variable to only consider two decimals after dot
    float   tempDVDD;       //used as intermediary variable to only consider two decimals after dot
    int     intermediary;   //used as intermediary variable to only consider two decimals after dot
    intermediary = *vref * 100;
    tempVRef = (float) intermediary / 100;
    intermediary = *dvdd * 100;
    tempDVDD = (float) intermediary / 100;
    *vset = (tempVRef* (*dacValue)) / 65535;
    if(*setValue)
    {
        AD5668_SetDac(*dacChanNumber, *dacValue);
    }
    if(*setCommand)
    {
        *cmdErr = AD5668_SetDacCommand(*cmdNumber, *cmdChannel, *cmdData);
        *setCommand = 0;
    }
    if (*ldac) //toggle the LDAC pin
    {
        AD5668_Ldac();
        *ldac=1;
    }
    if (*clr) //toggle the CLR pin
    {
        AD5668_Clear();
        *clr=1;
    }
    if(*loop)
    {
        for(i = 0; i < 8; i++)
        {
            rgADCValue[i] = AD5668_GetAdc(i);
            rgVRead[i] = tempDVDD * rgADCValue[i] / 4096;
        }
    }
    else
    {
        rgADCValue[*dacChanNumber] = AD5668_GetAdc(*dacChanNumber);
        rgVRead[*dacChanNumber] = tempDVDD * rgADCValue[*dacChanNumber] / 4096;
    }

    return(TRUE);
}
